--[[
The visual button for an instance of an action.  Based on settings, the button 
can look different.  It can look like a menu item, with the icon and spell name 
showing side by side, or look like a typical action button with just the icon 
showing.  Other appearances are possible as well.  This class controls 
implementing the standard appearance of the button, and how the visual effects 
actually appear (e.g., just the icon blinks, but the text does not, or the icon 
is highlighted, but the text is not).

An ActionButton instance is allocated for each action for a UnitGroupHeader.
Each ActionButton instance is associated with a single UnitGroupHeader and a 
single Action.

When an ActionButton is shown, it adjusts it's scale to the currently set scale,
then it tells the associated Action to identify the visual effects based on the 
state of the action and the unit the ActionButton is associated with.
--]]
uabActionButton = uab:NewModule("ActionButton", "Metrognome-2.0", "AceEvent-2.0")

uabActionButton.core = uab
uabActionButton.actionButtons = {}
uabActionButton.visibleButtons = {}

function uabActionButton:OnInitialize()
	uabActionButton.super.OnInitialize(self)
    self:RegisterMetro("ActionButtonUpdate", self.ActionButtonUpdate, 0, self)
    self:StartMetro("ActionButtonUpdate")
end

function uabActionButton:ActionButtonUpdate(elapsed)
    for button, _ in pairs(self.visibleButtons) do
        if not unit or button:IsUnit(unit) then
            local action = button:UnitStateModifiedAction()
            button:TimerEvent(elapsed, action)
            button:Update(action)
        end
    end
end

function uabActionButton:SetVisible(button)
    self.visibleButtons[button] = true
end

function uabActionButton:ClearVisible(button)
    self.visibleButtons[button] = nil
end

function uabActionButton:GetActionButtons()
    return self.actionButtons
end

function uabActionButton:RegisterActionButton(frame)
    self:TriggerEvent("UnitActionBars_ActionButtonCreated", frame)
    self.actionButtons[frame] = true
end

function uabActionButton:New(header, action, number)
    return self.core.classes.ActionButton:new(header, action, number)
end

function uabActionButton:NewCancel(header)
    return self.core.classes.ActionButtonNone:new(header)
end

uab.classes.ActionButton = uab.mixins.aceOO.Class(uab.mixins.aceEvent)

function uab.classes.ActionButton.prototype:init(header, action, number)
	uab.classes.ActionButton.super.prototype.init(self)
	
	self.header = header
    self.posstates = {}
    self.showstates = {}
    self.savedattrs = {}
    self.flashing = {}
    self.visualeffect = {}
    
    self.number = number

    self.framename = "uab_"..self.header.name.."_ActionBtn"..self.number
    self.timername = "ActionButtonTimer_"..self.framename
    self.maxalpha = 1.0

    --[[
	Create the action button frame
	--]]
    self.frame = getglobal(self.framename)
    if not self.frame then
        self.frame = CreateFrame("Button", self.framename, nil, "UAB_StandardActionButtonTemplate")
        
        uabActionButton:RegisterActionButton(self.frame)
    end
    
    self.frame:Hide()
    
    self.frame.icon = getglobal(self.frame:GetName().."Icon")
    self.frame.itemcount = getglobal(self.frame:GetName().."Count")
    self.frame.cooldown = getglobal(self.frame:GetName().."Cooldown")
    self.frame.border = getglobal(self.frame:GetName().."Border")
    self.frame.normaltexture = getglobal(self.frame:GetName().."NormalTexture")

    -- border fix-ups to smooth icon appearance a bit (borrowed from Bongos)
	self.frame.icon:SetTexCoord(0.06, 0.94, 0.06, 0.94)
        
    self.frame.cooldown:SetFrameLevel(2)

    self.frame.uabobj = self
    self.frame.cooldown:Show()
    self.frame.itemcount:Show()
    self.frame.border:Hide()

    self:HighlightBorderChanged()

    uabActionButton:SetBoolFrameAttribute(self.frame, "useparent-unit", true)
    uabActionButton:SetBoolFrameAttribute(self.frame, "useparent-unitsuffix", true)

    self:Enable()
    self:SetAction(action)

    self:SetParent()

    self.frame:SetScript("OnShow", function(self) self.uabobj:OnShow() end)
    self.frame:SetScript("OnHide", function(self) self.uabobj:OnHide() end)
    self.frame:SetScript("OnEnter", function(self, motion) self.uabobj:ShowTooltip() end)
    self.frame:SetScript("OnLeave", function(self, motion) GameTooltip:Hide() end)

    --[[
    Start out with the button always hidden.  When some frame registers for the
    unit group of the activator for this button, the display properties of this
    button will get adjusted then.  We just need to start by indicating that the
    button is always hidden.
    --]]
    self:PrepareForLayouts()
    self:LayoutsDone()
end

function uab.classes.ActionButton.prototype:PrepareForLayouts()
    for idx, _ in pairs(self.showstates) do
        self.showstates[idx] = nil
    end

    for x, _ in pairs(self.posstates) do
        self.posstates[x].pt = nil
        self.posstates[x].x = nil
        self.posstates[x].y = nil
        self.posstates[x] = nil
    end
end

function uab.classes.ActionButton.prototype:AddShowStates(states)
    self.showstates[states] = true
end

function uab.classes.ActionButton.prototype:SetPointForStates(states, ofspoint, ofsx, ofsy)
    self.posstates[states] = { pt = ofspoint, x = ofsx, y = ofsy }
end

function uab.classes.ActionButton.prototype:LayoutsDone()
    local ptattr, xattr, yattr
    
    for posstates, pos in pairs(self.posstates) do
        if not ptattr then
            ptattr = ""
            xattr = ""
            yattr = ""
        else
            ptattr = ptattr .. ";"
            xattr = xattr .. ";"
            yattr = yattr .. ";"
        end

        ptattr = ptattr .. posstates .. ":" .. pos.pt
        xattr = xattr .. posstates .. ":" .. pos.x
        yattr = yattr .. posstates .. ":" .. pos.y
    end

    uabActionButton:SetFrameAttribute(self.frame, "ofspoint", ptattr)
    uabActionButton:SetFrameAttribute(self.frame, "ofsx", xattr)
    uabActionButton:SetFrameAttribute(self.frame, "ofsy", yattr)

    local showstates

    for states, _ in pairs(self.showstates) do
        if not showstates then
            showstates = ""
        else
            showstates = showstates .. ","
        end

        showstates = showstates .. states
    end

    if not showstates then
        uabActionButton:SetFrameAttribute(self.frame, "showstates", nil)
        uabActionButton:SetFrameAttribute(self.frame, "hidestates", "*")
    else
        uabActionButton:SetFrameAttribute(self.frame, "showstates", showstates)
        uabActionButton:SetFrameAttribute(self.frame, "hidestates", nil)
    end
end

function uab.classes.ActionButton.prototype:SetParent()
    if self.header.frame then
        self.frame:SetParent(self.header.frame)
        uabActionButton:SetFrameFrameAttribute(self.header.frame, "addchild", self.frame)
    end
end

function uab.classes.ActionButton.prototype:SetAction(action)
    if self.action then
        self.action:RemoveButton(self)
    end
    self.action = action
    self.actiondisabled = nil
    if self.action then
        self.action:AddButton(self)
    end
    
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:SetHostileAction(action)
    if self.hostileaction then
        self.hostileaction:RemoveButton(self)
    end
    self.hostileaction = action
    self.hostileactiondisabled = nil
    if self.hostileaction then
        self.hostileaction:AddButton(self)
    end
        
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:ClearAttributes(attributes)
	for name, _ in pairs(attributes) do
		if self.savedattrs[name] then
			uabActionButton:SetFrameAttribute(self.frame, name, self.savedattrs[name])
		else
			uabActionButton:SetFrameAttribute(self.frame, name, nil)
		end
	end
end

function uab.classes.ActionButton.prototype:SetAttributes(attributes)
	for name, val in pairs(attributes) do
		self.savedattrs[name] = self.frame:GetAttribute(name)
		uabActionButton:SetFrameAttribute(self.frame, name, val)
	end
end

function uab.classes.ActionButton.prototype:CurrentUnit()
    return SecureButton_GetModifiedUnit(self:Frame())
end

function uab.classes.ActionButton.prototype:UnitStateModifiedAction()
    local unit = self:CurrentUnit()
    local action = self.action
    
    if unit and UnitCanAttack("player", unit) and self.header.group.activator:HasHostileActions() then
        action = self.hostileaction
    end
    
    return action
end

function uab.classes.ActionButton.prototype:Show()
    local action = self:UnitStateModifiedAction()
                 
    if not action or not action:GetData() then
        self.frame:SetAlpha(0)
    else
        self:StatusChanged(action)
    end
end

function uab.classes.ActionButton.prototype:Hide()
	self.frame:Hide()
end

function uab.classes.ActionButton.prototype:DisableAction()
    self.actiondisabled = true
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:DisableHostileAction()
    self.hostileactiondisabled = true
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:ActionDisabled()
    if self.actiondisabled then
        return true
    end
    
    if not self.action or not self.action:GetData() then
        return true
    end
end

function uab.classes.ActionButton.prototype:HostileActionDisabled()
    if self.hostileactiondisabled then
        return true
    end
    
    if not self.hostileaction or not self.hostileaction:GetData() then
        return true
    end
end

function uab.classes.ActionButton.prototype:Enable()
    self.enabled = true
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:Disable()
    self.enabled = nil
    self:CheckEnabledState()
end

function uab.classes.ActionButton.prototype:CheckEnabledState()
    if self.enabled and not (self:ActionDisabled() and self:HostileActionDisabled()) then
        self:DoEnable()
    else
        self:DoDisable()
    end
end

function uab.classes.ActionButton.prototype:DoEnable()
    if not self.isenabled then
        self.isenabled = true
        self:StayOpenChanged()
    end
end

function uab.classes.ActionButton.prototype:DoDisable()
    if self.isenabled then
        self.isenabled = nil
        self:StayOpenChanged()    
    end
end

function uab.classes.ActionButton.prototype:StayOpenChanged()
    if not self.header.group:IsStayOpen() or not self.isenabled then
        uabActionButton:SetFrameAttribute(self.frame, "newstate", "*:0")
    else
        uabActionButton:SetFrameAttribute(self.frame, "newstate", nil)
    end
end

function uab.classes.ActionButton.prototype:HighlightBorderChanged()
    local width = uab.db.profile.borderwidth
     
    if width > 5 then
        width = 5
    elseif width < 2 then
        width = 2
    end
    
    self.frame.border:SetTexture("Interface\\AddOns\\UnitActionBars\\images\\swborder" .. width)
end

function uab.classes.ActionButton.prototype:Frame()
	return self.frame
end

function uab.classes.ActionButton.prototype:BorderColor(r, g, b, a)
    if a == 0 then
        self.frame.border:Hide()
    else
        self.frame.border:SetVertexColor(r,g,b,a)
        self.frame.border:Show()
    end
end

function uab.classes.ActionButton.prototype:SetAlpha(alpha)
    self.maxalpha = alpha
end

function uab.classes.ActionButton.prototype:StartFlash()
    if not self.flashing.enabled then
        self.flashing.timer = uab.db.profile.blinkcycle/2
        self.flashing.direction = 1
        self.flashing.enabled = true
    end
end

function uab.classes.ActionButton.prototype:EndFlash()
    if self.flashing.enabled then
        self.flashing.enabled = nil
    end
end

function uab.classes.ActionButton.prototype:StatusChanged(action)
    if action then
        action:ShowButton(self)
    else
        self.frame:SetAlpha(0)
    end
end

function uab.classes.ActionButton.prototype:Update(action)
    if not action then
        action = self:UnitStateModifiedAction()
    end
    
    self:UpdateCooldown(action)
    self:StatusChanged(action)
end

function uab.classes.ActionButton.prototype:UpdateCooldown(action)
    if not action then
        action = self:UnitStateModifiedAction()
    end
    
    if action and action:GetData() then
        local start, duration, enable = action:GetData():GetCooldown()

        if start and start > 0 and duration and duration > 0 and enable and enable > 0 then
            if not self.oncooldown or self.oncooldown ~= action then
                CooldownFrame_SetTimer(self.frame.cooldown, start, duration, enable)
                self.oncooldown = action
            end
        else
            CooldownFrame_SetTimer(self.frame.cooldown, 0, 0, 0)
            self.oncooldown = nil
        end
    elseif self.oncooldown then
        CooldownFrame_SetTimer(self.frame.cooldown, 0, 0, 0)
        self.oncooldown = nil
    end
end

function uab.classes.ActionButton.prototype:UpdateUsable(action)
    if not action then
        action = self:UnitStateModifiedAction()
    end
    
    if action and action:GetData() then
        action:UpdateVisualEffects(self)
    else
        self.frame:SetAlpha(0)
    end
end

function uab.classes.ActionButton.prototype:TimerEvent(elapsed, action)
    if self.isenabled then
        local a = self.maxalpha
        if not action then
            action = self:UnitStateModifiedAction()
        end
    
        if action and action:GetData() then
    	    if self.flashing.enabled then
          	    self.flashing.timer = self.flashing.timer - elapsed
    	        if self.flashing.timer < 0 then
          	        self.flashing.timer = uab.db.profile.blinkcycle/2
                	if self.flashing.direction then
    	                self.flashing.direction = nil
          	        else
                	    self.flashing.direction = 1
    	            end
          	    end
            
                if self.flashing.direction then
                    a = ((self.flashing.timer * 2)/uab.db.profile.blinkcycle)*self.maxalpha
    	        else
                	a = (1 - ((self.flashing.timer * 2)/uab.db.profile.blinkcycle))*self.maxalpha
          	    end
    	    end
        else
    	    a = 0
    	    self.flashing.enabled = nil
        end

        self.frame:SetAlpha(a)
    
        --Tooltip stuff, probably for the cooldown timer
        if self.updateTooltip then
            self.updateTooltip = self.updateTooltip - elapsed
            if self.updateTooltip <= 0 then
                if GameTooltip:IsOwned(self) then
                    self:ShowTooltip(action)
                end
                self.updateTooltip = TOOLTIP_UPDATE_TIME
            end
        end
    end
end

function uab.classes.ActionButton.prototype:IsUnit(unit)
	return unit == SecureButton_GetModifiedUnit(self.frame)
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.ActionButton.prototype:ShowTooltip(action)
    if not action then
        action = self:UnitStateModifiedAction()
    end
             
    if action and action:GetData() then
        action:ShowTooltip(self.frame)
    end
end

function uab.classes.ActionButton.prototype:OnShow()
    self.isvisible = true
    uabActionButton:SetVisible(self)
    self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
end

function uab.classes.ActionButton.prototype:OnHide()
    self.isvisible = nil
    uabActionButton:ClearVisible(self)
end

