--[[
A SecureHeader presents an interface to the the Blizzard/WoW secure header
frame for the functionality that UAB needs.
--]]
uab.classes.SecureHeader = uab.mixins.aceOO.Class()
uab.classes.SecureHeader.headers = {}

function uab.classes.SecureHeader:GetHeader(context)
	if not self.headers[context] then
		self.headers[context] = self:new(context)
	end

	return self.headers[context]	
end

function uab.classes.SecureHeader:HideAllHeaders()
    for _, header in pairs(self.headers) do
        uab.SetFrameAttribute(header.frame, "state", "1")
        uab.SetFrameAttribute(header.frame, "state", "0")
    end
end

function uab.classes.SecureHeader.prototype:init(context)
	uab.classes.SecureHeader.super.prototype.init(self)

	self.context = context
    self.stateparent = {}
    self.stateparentrange = {}
    self.statescale = {}
    self.statescalerange = {}
    self.statepos = {}
    self.stateposrange = {}
    self.statemapanchor = {}

	--[[
	Create the header frame.
	--]]

    self.framename = "uab_"..context.."_SecureHeader"
    self.frame = getglobal(self.framename)
    if not self.frame then
        self.frame = CreateFrame("Frame", self.framename, UIParent, "SecureStateHeaderTemplate")
    end
    
    self.frame.uabobj = self
    uab.SetFrameAttribute(self.frame, "state", "0")

    uab.SetBoolFrameAttribute(self.frame, "useparent-unit", true)
    uab.SetBoolFrameAttribute(self.frame, "useparent-unitsuffix", true)
    uab.SetFrameAttribute(self.frame, "statemap-anchor", "$input")

    self:SetStatePosition()
    self:SetStateScale()
end

function uab.classes.SecureHeader.prototype:SetAnchorState(state, rule)
    self.statemapanchor[state] = rule
    uab.SetFrameAttribute(self.frame, "statemap-anchor-"..state, rule)
end

function uab.classes.SecureHeader.prototype:IAmDead()
    --[[
    Set the header frame state attribute to 0 to (hopefully) cause our cancel and action buttons
    to hide (because of the state change to a hidden state).  Since we are dead, we are no longer
    in combat, so this *should* be ok to do.
    --]]
    uab.SetFrameAttribute(self.frame, "state", "0")
end

function uab.classes.SecureHeader.prototype:SetStateScale(state, scale)
    if state then
        if scale then
            self.statescale[state] = scale
        else
            self.statescale[state] = nil
        end
    end
    
    self:SetStateScaleAttributes()    
end

function uab.classes.SecureHeader.prototype:SetStateScaleRange(minstate, maxstate, scale)
    if minstate and maxstate then
        local idxtodel
        for idx, range in pairs(self.statescalerange) do
            if range.minstate == minstate and range.maxstate == maxstate then
                idxtodel = idx
                break
            end
        end

        if idxtodel then
            self.statescalerange[idxtodel] = nil
        end

        if scale then
            table.insert(self.statescalerange, { minstate = minstate, maxstate = maxstate, scale = scale })
        end
    end
    
    self:SetStateScaleAttributes()    
end

function uab.classes.SecureHeader.prototype:SetStatePosition(state, x, y, relpoint, point)
    local headofspoint
    local headofsrelpoint
    local headofsx
    local headofsy

    if state then
        if x and y and relpoint and point then
            self.statepos[state] = {}
            self.statepos[state].point = point
            self.statepos[state].relpoint = relpoint
            self.statepos[state].x = x
            self.statepos[state].y = y
        else
            self.statepos[state] = nil
        end
    end

    self:SetStatePositionAttributes()
end

function uab.classes.SecureHeader.prototype:SetStatePositionRange(minstate, maxstate, x, y, relpoint, point)
    local headofspoint
    local headofsrelpoint
    local headofsx
    local headofsy

    if minstate and maxstate then
        local idxtodel
        for idx, range in pairs(self.stateposrange) do
            if range.minstate == minstate and range.maxstate == maxstate then
                idxtodel = idx
                break
            end
        end

        if idxtodel then
            self.stateposrange[idxtodel] = nil
        end

        if x and y and relpoint and point then
            table.insert(self.stateposrange, { minstate = minstate, maxstate = maxstate, point = point, relpoint = relpoint, x = x, y = y })
        end
    end

    self:SetStatePositionAttributes()
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.SecureHeader.prototype:SetStateScaleAttributes()
    local headscale

    for state, statescale in pairs(self.statescale) do
        if not headscale then
            headscale = ""
        else
            headscale = headscale .. ";"
        end

        headscale = headscale .. state .. ":" .. statescale
    end
    
    for _, range in pairs(self.statescalerange) do
        if not headscale then
            headscale = ""
        else
            headscale = headscale .. ";"
        end

        headscale = headscale .. range.minstate .. "-" .. range.maxstate .. ":" .. range.scale
    end

    local defaultscale
    if self.context == "Global" then
        defaultscale = uab.db.profile.scale.default.value
    else
        defaultscale = uab.db.prifile.scale[self.context].value
    end
        
    if headscale then
        headscale = headscale .. ";*:" .. defaultscale
    else
        headscale = "*:" .. defaultscale
    end

    uab.SetFrameAttribute(self.frame, "headscale", headscale)
end

function uab.classes.SecureHeader.prototype:SetStatePositionAttributes()
    local headofspoint
    local headofsrelpoint
    local headofsx
    local headofsy

    for state, pos in pairs(self.statepos) do
        if not headofspoint then
            headofspoint = ""
            headofsrelpoint = ""
            headofsx = ""
            headofsy = ""
        else
            headofspoint = headofspoint .. ";"
            headofsrelpoint = headofsrelpoint .. ";"
            headofsx = headofsx .. ";"
            headofsy = headofsy .. ";"
        end

        headofspoint = headofspoint .. state .. ":" .. pos.point
        headofsrelpoint = headofsrelpoint .. state .. ":" .. pos.relpoint
        headofsx = headofsx .. state .. ":" .. pos.x
        headofsy = headofsy .. state .. ":" .. pos.y
    end

    for _, range in pairs(self.stateposrange) do
        if not headofspoint then
            headofspoint = ""
            headofsrelpoint = ""
            headofsx = ""
            headofsy = ""
        else
            headofspoint = headofspoint .. ";"
            headofsrelpoint = headofsrelpoint .. ";"
            headofsx = headofsx .. ";"
            headofsy = headofsy .. ";"
        end

        local staterange = range.minstate .. "-" .. range.maxstate
        
        headofspoint = headofspoint .. staterange .. ":" .. range.point
        headofsrelpoint = headofsrelpoint .. staterange .. ":" .. range.relpoint
        headofsx = headofsx .. staterange .. ":" .. range.x
        headofsy = headofsy .. staterange .. ":" .. range.y
    end

    local defaultpoint
    local defaultrelpoint
    local defaultx
    local defaulty
    
    if self.context == "Global" then
        defaultpoint = uab.db.profile.position.default.point
        defaultrelpoint = uab.db.profile.position.default.relpoint
        defaultx = uab.db.profile.position.default.x
        defaulty = uab.db.profile.position.default.y
    else
        defaultpoint = uab.db.profile.position[self.context].point
        defaultrelpoint = uab.db.profile.position[self.context].relpoint
        defaultx = uab.db.profile.position[self.context].x
        defaulty = uab.db.profile.position[self.context].y
    end
        
    if headofspoint then
        headofspoint = headofspoint .. ";*:" .. defaultpoint
        headofsrelpoint = headofsrelpoint .. ";*:" .. defaultrelpoint
        headofsx = headofsx .. ";*:" .. defaultx
        headofsy = headofsy .. ";*:" .. defaulty
    else
        headofspoint = "*:" .. defaultpoint
        headofsrelpoint = "*:" .. defaultrelpoint
        headofsx = "*:" .. defaultx
        headofsy = "*:" .. defaulty
    end

    uab.SetFrameAttribute(self.frame, "headofspoint", headofspoint)
    uab.SetFrameAttribute(self.frame, "headofsrelpoint", headofsrelpoint)
    uab.SetFrameAttribute(self.frame, "headofsx", headofsx)
    uab.SetFrameAttribute(self.frame, "headofsy", headofsy)
end

